@@ -583,14 +583,44 @@ public class ComplexDoubleMatrix {
 	/** Return transposed copy of this matrix */
 	public ComplexDoubleMatrix transpose() {
 		ComplexDoubleMatrix result = new ComplexDoubleMatrix(columns, rows);
-		
+
+                ComplexDouble c = new ComplexDouble(0);
+
 		for (int i = 0; i < rows; i++)
 			for (int j = 0; j < columns; j++)
-				result.put(j, i, get(i, j));
+				result.put(j, i, get(i, j, c));
 		
 		return result;
 	}
-	
+
+        public ComplexDoubleMatrix hermitian() {
+            ComplexDoubleMatrix result = new ComplexDoubleMatrix(columns, rows);
+
+            ComplexDouble c = new ComplexDouble(0);
+
+            for (int i = 0; i < rows; i++)
+                for (int j = 0; j < columns; j++)
+                    result.put(j, i, get(i, j, c).conji());
+            return result;
+        }
+
+        /**
+         * Compute complex conjugate (in-place).
+         */
+        public ComplexDoubleMatrix conji() {
+            ComplexDouble c = new ComplexDouble(0.0);
+            for (int i = 0; i < length; i++)
+                put(i, get(i, c).conji());
+            return this;
+        }
+
+        /**
+         * Compute complex conjugate.
+         */
+        public ComplexDoubleMatrix conj() {
+            return dup().conji();
+        }
+
 		
 	/** Compare two matrices.
 	 * @param o Object to compare to
@@ -687,7 +717,7 @@ public class ComplexDoubleMatrix {
 	public ComplexDoubleMatrix dup() {
 		ComplexDoubleMatrix out = new ComplexDoubleMatrix(rows, columns);
 
-                System.arraycopy(out.data, 0, data, 0, 2 * length);
+                JavaBlas.rcopy(2*length, data, 0, 1, out.data, 0, 1);
 		
 		return out;
 	}
@@ -708,7 +738,13 @@ public class ComplexDoubleMatrix {
 		return this;
 	}
 
-	public ComplexDoubleMatrix put(int rowIndex, int columnIndex, ComplexDouble value) {
+	public ComplexDoubleMatrix put(int rowIndex, int columnIndex, double realValue, double complexValue) {
+		data[2*index(rowIndex, columnIndex)] =  realValue;
+		data[2*index(rowIndex, columnIndex)+1] =  complexValue;
+		return this;
+	}
+
+        public ComplexDoubleMatrix put(int rowIndex, int columnIndex, ComplexDouble value) {
 		int i = 2*index(rowIndex, columnIndex);
 		data[i] = value.real(); data[i+1] = value.imag();
 		return this;
@@ -729,6 +765,11 @@ public class ComplexDoubleMatrix {
             int i = 2*index(rowIndex, columnIndex);
             return new ComplexDouble(data[i], data[i+1]);
 	}
+
+        /** Get matrix element, passing the variable to store the result. */
+        public ComplexDouble get(int rowIndex, int columnIndex, ComplexDouble result) {
+            return get(index(rowIndex, columnIndex));
+        }
 	
 	public DoubleMatrix getReal() {
 		DoubleMatrix result = new DoubleMatrix(rows, columns);
@@ -764,6 +805,12 @@ public class ComplexDoubleMatrix {
 		data[2*i] = v;
 		return this;
 	}
+
+        public ComplexDoubleMatrix put(int i, double r, double c) {
+            data[2*i] = r;
+            data[2*i+1] = c;
+            return this;
+        }
 	
 	public ComplexDoubleMatrix put(int i, ComplexDouble v) {
 		data[2*i] = v.real();
@@ -1194,13 +1241,6 @@ public class ComplexDoubleMatrix {
 	public ComplexDoubleMatrix truth() {
 		return dup().truthi();
 	}
-        
-        public ComplexDoubleMatrix conji() {
-            ComplexDouble c = new ComplexDouble(0.0);
-            for (int i = 0; i < length; i++)
-                put(i, get(i, c).conji());
-            return this;
-        }
 
 	/****************************************************************
 	 * Rank one-updates
@@ -1257,12 +1297,12 @@ public class ComplexDoubleMatrix {
 		return sum().div((double)length);
 	}
 	
-	/* computes this^T * other */
+	/** Computes this^T * other */
 	public ComplexDouble dotc(ComplexDoubleMatrix other) {
 		return SimpleBlas.dotc(this, other);
 	}
 	
-	/* computs this^H * other */
+	/** Computes this^H * other */
 	public ComplexDouble dotu(ComplexDoubleMatrix other) {
 		return SimpleBlas.dotu(this, other);
 	}
