@@ -583,14 +583,44 @@ public class ComplexFloatMatrix {
 	/** Return transposed copy of this matrix */
 	public ComplexFloatMatrix transpose() {
 		ComplexFloatMatrix result = new ComplexFloatMatrix(columns, rows);
-		
+
+                ComplexFloat c = new ComplexFloat(0);
+
 		for (int i = 0; i < rows; i++)
 			for (int j = 0; j < columns; j++)
-				result.put(j, i, get(i, j));
+				result.put(j, i, get(i, j, c));
 		
 		return result;
 	}
-	
+
+        public ComplexFloatMatrix hermitian() {
+            ComplexFloatMatrix result = new ComplexFloatMatrix(columns, rows);
+
+            ComplexFloat c = new ComplexFloat(0);
+
+            for (int i = 0; i < rows; i++)
+                for (int j = 0; j < columns; j++)
+                    result.put(j, i, get(i, j, c).conji());
+            return result;
+        }
+
+        /**
+         * Compute complex conjugate (in-place).
+         */
+        public ComplexFloatMatrix conji() {
+            ComplexFloat c = new ComplexFloat(0.0f);
+            for (int i = 0; i < length; i++)
+                put(i, get(i, c).conji());
+            return this;
+        }
+
+        /**
+         * Compute complex conjugate.
+         */
+        public ComplexFloatMatrix conj() {
+            return dup().conji();
+        }
+
 		
 	/** Compare two matrices.
 	 * @param o Object to compare to
@@ -687,7 +717,7 @@ public class ComplexFloatMatrix {
 	public ComplexFloatMatrix dup() {
 		ComplexFloatMatrix out = new ComplexFloatMatrix(rows, columns);
 
-                System.arraycopy(out.data, 0, data, 0, 2 * length);
+                JavaBlas.rcopy(2*length, data, 0, 1, out.data, 0, 1);
 		
 		return out;
 	}
@@ -708,7 +738,13 @@ public class ComplexFloatMatrix {
 		return this;
 	}
 
-	public ComplexFloatMatrix put(int rowIndex, int columnIndex, ComplexFloat value) {
+	public ComplexFloatMatrix put(int rowIndex, int columnIndex, float realValue, float complexValue) {
+		data[2*index(rowIndex, columnIndex)] =  realValue;
+		data[2*index(rowIndex, columnIndex)+1] =  complexValue;
+		return this;
+	}
+
+        public ComplexFloatMatrix put(int rowIndex, int columnIndex, ComplexFloat value) {
 		int i = 2*index(rowIndex, columnIndex);
 		data[i] = value.real(); data[i+1] = value.imag();
 		return this;
@@ -729,6 +765,11 @@ public class ComplexFloatMatrix {
             int i = 2*index(rowIndex, columnIndex);
             return new ComplexFloat(data[i], data[i+1]);
 	}
+
+        /** Get matrix element, passing the variable to store the result. */
+        public ComplexFloat get(int rowIndex, int columnIndex, ComplexFloat result) {
+            return get(index(rowIndex, columnIndex));
+        }
 	
 	public FloatMatrix getReal() {
 		FloatMatrix result = new FloatMatrix(rows, columns);
@@ -764,6 +805,12 @@ public class ComplexFloatMatrix {
 		data[2*i] = v;
 		return this;
 	}
+
+        public ComplexFloatMatrix put(int i, float r, float c) {
+            data[2*i] = r;
+            data[2*i+1] = c;
+            return this;
+        }
 	
 	public ComplexFloatMatrix put(int i, ComplexFloat v) {
 		data[2*i] = v.real();
@@ -1194,13 +1241,6 @@ public class ComplexFloatMatrix {
 	public ComplexFloatMatrix truth() {
 		return dup().truthi();
 	}
-        
-        public ComplexFloatMatrix conji() {
-            ComplexFloat c = new ComplexFloat(0.0f);
-            for (int i = 0; i < length; i++)
-                put(i, get(i, c).conji());
-            return this;
-        }
 
 	/****************************************************************
 	 * Rank one-updates
@@ -1257,12 +1297,12 @@ public class ComplexFloatMatrix {
 		return sum().div((float)length);
 	}
 	
-	/* computes this^T * other */
+	/** Computes this^T * other */
 	public ComplexFloat dotc(ComplexFloatMatrix other) {
 		return SimpleBlas.dotc(this, other);
 	}
 	
-	/* computs this^H * other */
+	/** Computes this^H * other */
 	public ComplexFloat dotu(ComplexFloatMatrix other) {
 		return SimpleBlas.dotu(this, other);
 	}
